/* lw-support/src/lib/Events/CallbackUpdate.h
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



/*! \brief Update event callback object.

Using lw::EventManager's update mechanism, objects can (in the event loop) register themselves as
requiring an update. After the event loop has completed, all objects needing an update will then be
processed. During this processing stage, it is safe for objects to delete themselves or other
objects (which may also be on the update list). The update function will be called at most once
per event loop, and only if the object has requested that it be added to the update list.

It is possible for an object to be registered with more than one lw::EventManager's update list at
a time; but it will only be called a maximum of once per call to lw::EventManager::processEvents(),
regardless of how many times it is registered. Deleting an object which is registered with an
lw::EventManager will cause the object to become unregisterd.

*/
class EventCallbackUpdate {
public:
    /// Destructor. Unregisters the device from any update lists it may be in.
    virtual ~EventCallbackUpdate();

    /// Update callback. The object can safely delete itself in here.
    virtual void update(void) throw() = 0;

private:
    friend class EventManager;
    std::vector<lw::EventManager*> managers;
};



}

/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
*/
