/* lw-support/src/lib/Exceptions/BadTime.h
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



/// Base class of all time-related exceptions.
class BadTime : public Exception {
public:
    /// Constructor.
    BadTime(const std::wstring& errorDesc)
        : Exception(errorDesc)
    { }

    /// Destructor.
    virtual ~BadTime() throw()
    { }
};



/*! \brief Thrown when a time operation overflows.

This exception will be thrown when a subtraction operation on an elapsed
time period would have resulted in a negative time period. Since this
result does not make sense, the operation cannot proceed.

*/
class BadTimeOverflow : public BadTime {
public:
    /// Constructor.
    BadTimeOverflow() : BadTime(L"Cannot represent negative time periods.")
    { }
};



/*! \brief Thrown when a time value is bad.

This exception is thrown when a time value is bad (for instance, you are
trying to specify a negative period, or a minute of 61, etc.).

*/
class BadTimeValue : public BadTime {
public:
    /// Constructor.
    BadTimeValue() : BadTime(L"Bad time value.")
    { }
};



/// Thrown when a bad date is specified.
class BadDate : public BadTime {
public:
    BadDate() : BadTime(L"Bad date specified.")
    { }
};



}
