/* lw-support/src/lib/Exceptions/DNS.h
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



/*! \brief Thrown when a DNS error occurs.

This class is thrown when a DNS lookup fails. It contains the name
of the host being looked up, and a description of what failed.

*/
class DnsError : public Exception {
private:
    std::wstring reason;
    std::string host;

    static std::wstring buildDesc(const std::wstring& reason,
        const std::string& host);

public:
    /*! \brief Constructor.

    \param reason A description of what failed.
    \param host The host name being manipulated.

    */
    DnsError(const std::wstring& reason,
        const std::string& host);

    /// Destructor.
    virtual ~DnsError() throw()
    { }

    /// Get reason for failure.
    const std::wstring& getReason() const
    { return reason; }

    /// Get host name being looked up.
    const std::string& getHost() const
    { return host; }
};



}
