/* lw-support/src/lib/Exceptions/FIFO.h
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



/// FIFO exception base class.
class FIFOException : public Exception {
public:
    /// Constructor.
    FIFOException(const std::wstring& desc)
        : Exception(desc)
    { }
};



/// FIFO memory limit exceeded.
class FIFOOverflow : public FIFOException {
public:
    /// Constructor.
    FIFOOverflow()
        : FIFOException(L"FIFO buffer overflowed (too much data).")
    { }
};



/// Ran out of system memory while enlarging FIFO.
class FIFOOutOfMemory : public FIFOException {
private:
    static std::wstring getDesc(size_t amt);
    size_t amt;

public:
    /// Constructor.
    FIFOOutOfMemory(size_t amt)
        : FIFOException(getDesc(amt)), amt(amt)
    { }

    /// Get amount allocated, in bytes.
    size_t getAmount() const
    { return amt; }
};



/// Not enough data in FIFO.
class FIFOUnderflow : public FIFOException {
public:
    /// Constructor.
    FIFOUnderflow()
        : FIFOException(L"FIFO buffer underflowed (not enough data).")
    { }
};



}
