/* lw-support/src/lib/Exceptions/IO.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



std::wstring IOModeError::getModeDesc(IOMode mode)
{
    switch(mode) {
        case IOClosed:
            return L"closed";
        case IONone:
            return L"not opened for reading or writing";
        case IORead:
            return L"opened for reading";
        case IOWrite:
            return L"opened for writing";
        case IOReadWrite:
            return L"opened for reading and writing";
    }

    // shouldn't reach here
    return L"Bug in IOModeError::getModeDesc()";
}



std::wstring IOModeError::getDesc(IOMode currentMode, IOMode requiredMode,
    const std::wstring& op)
{
    std::wostringstream o;

    o << L"Could not perform operation on device in its current mode.\n"
         L"  Current mode : " << getModeDesc(currentMode) << L"\n"
         L"  Required mode: " << getModeDesc(requiredMode) << L"\n"
         L"  Operation    : " << op;
    return o.str();
}



IOModeError::IOModeError(IOInterface* ioInterface, const std::wstring& op,
    IOMode requiredMode)
    : IOException(getDesc(ioInterface->getIOMode(), requiredMode, op),
        ioInterface),
      op(op), currentMode(ioInterface->getIOMode()),
      requiredMode(requiredMode)
{
}



std::wstring IOTimeout::getDesc(const std::wstring& op, int timeout,
    IOTimeoutMode timeoutMode, size_t amountTransferred)
{
    std::wostringstream o;

    o << L"I/O operation timed out.\n"
         L"  Operation   : " << op << L"\n"
         L"  Timeout     : " << timeout;
    if(timeout < 0) o << L" (never -- so why are we here?)";
    else if(!timeout) o << L" (immediately)";
    else o << "ms";

    o << L"\n  Timeout mode: ";
    switch(timeoutMode) {
        case IOTimeoutHardPartial:
            o << L"hard timeout, partial transfer OK";
            break;

        case IOTimeoutHardFull:
            o << L"hard timeout, full transfer only";
            break;

        case IOTimeoutSoftPartial:
            o << L"soft timeout, partial transfer OK";

        case IOTimeoutSoftFull:
            o << L"soft timeout, full transfer only";
            break;

        case IOTimeoutOneShot:
            o << L"one-shot";
            break;
    }

    o << L"\n  Bytes transferred before timeout: " << amountTransferred;

    return o.str();
}



IOTimeout::IOTimeout(IOInterface* ioInterface, const std::wstring& op,
    int timeout, IOTimeoutMode timeoutMode, size_t amountTransferred)
    : IOException(getDesc(op, timeout, timeoutMode, amountTransferred),
        ioInterface),
      op(op), timeout(timeout), timeoutMode(timeoutMode),
      amountTransferred(amountTransferred)
{
}



std::wstring FileNotFound::getDesc(const char* fileName)
{
    std::wostringstream o;

    o << L"File not found.\n"
         L"  Filename: ";
    if(fileName) o << utf8ToUcs4(fileName, true);
    else o << L"(null)";

    return o.str();
}



FileNotFound::FileNotFound(const char* fileName)
    : Exception(getDesc(fileName)), fileName(fileName)
{
}



FileNotFound::~FileNotFound() throw()
{
}



}
