/* lw-support/src/lib/Exceptions/NullPointer.h
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



/*! \brief Null pointer exception.

This exception is used whenever a function that is expecting a valid pointer is passed a null
pointer.

*/
class NullPointer : public Exception {
public:
    /// Construct from pointer name.
    NullPointer(const std::wstring& pointerName)
        : lw::Exception(L"Null pointer exception: " + pointerName), pointerName(pointerName)
    { }

    /// Destructor (does nothing).
    virtual ~NullPointer() throw()
    { }

    /// Name of the pointer causing the problem.
    const std::wstring pointerName;

    /*! \brief Check a pointer.

    \param ptr The pointer to check.
    \param name Descriptive name of the pointer.
    \param func Name of the function, passed to chain(). May be omitted.

    If \a ptr is a null (zero) pointer, this function will throw a NullPointer exception.
    Otherwise, it does nothing.

    */
    inline static void check(const void* ptr, const wchar_t* name, const wchar_t* func = 0)
    {
        if(!ptr) {
            if(func) throw NullPointer(name).chain(func);
            throw NullPointer(name);
        }
    }

};



}
