/* lw-support/src/lib/Exceptions/ParseError.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



std::wstring ParseError::constructDesc(const std::wstring& reason,
    const std::wstring& src, std::wstring::size_type errorPos)
{
    std::wostringstream o;
    std::wstring::size_type len = src.length();

    if(!len) {
        o << "Error while parsing input.\n"
            " Reason: " << reason << "\n"
            " (std::string is empty)";
        return o.str();
    }

    std::wstring::size_type start = std::max(errorPos, std::wstring::size_type(32)) - 32,
        end = std::min(errorPos + 32, len);

    o << "Error while parsing input.\n"
         " Reason      : " << reason << "\n"
         " Char offset : " << errorPos << "\n"
         " Nearby      : " << src.substr(start, end) << "\n"
         " Error at    : " << std::wstring(errorPos - start, L' ') << '^';
    return o.str();
}



ParseError::ParseError(const std::wstring& reason,
    const std::wstring& src, std::wstring::size_type errorPos)
    : Exception(constructDesc(reason, src, errorPos)),
      reason(reason), src(src), errorPos(errorPos)
{
}



}
