/* lw-support/src/lib/Exceptions/ParseError.h
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



/*! \brief Exception thrown when parsing a std::string fails.

This exception should be thrown when the parsing of a std::string fails. It
is intended for std::strings which are "displayable", since the exception
message will contain a copy of (part of) the std::string being parsed. The
exception message is intended for output onto a fixed-width font
display.

*/
class ParseError : public Exception {
private:
    std::wstring reason, src;
    std::wstring::size_type errorPos;

    static std::wstring constructDesc(const std::wstring& reason,
        const std::wstring& src, std::wstring::size_type errorPos);

public:
    /*! \brief Constructor.

    \param reason A description of why the parsing error occured.
    \param src The source std::string.
    \param errorPos The character offset at which the error occured.

    */
    ParseError(const std::wstring& reason,
        const std::wstring& src, std::wstring::size_type errorPos);

    /// Destructor.
    virtual ~ParseError() throw()
    { }

    /// Gets the reason for the parsing error.
    const std::wstring& getReason() const
    { return reason; }

    /// Gets the source std::string.
    const std::wstring& getSource() const
    { return src; }

    /// Gets the position (in characters) at which the error occured.
    std::wstring::size_type getErrorPos() const
    { return errorPos; }
};



}
