/* lw-support/src/lib/Exceptions/SystemError.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



std::wstring SystemError::errnoToString(int errnum)
{
    std::wostringstream o;
    o << L"System error.\n"
         L"  Error number: " << errnum << L"\n"
         L"  Description : ";

#if 0
    // this is needed if we're using the non-GNU version of strerror_r()
    // specified by POSIX, with signature:
    //    int strerror_r(int, char*, int)
    //
    // the GNU version may go away one day, according to the manpage.
    int bufSize = 500;
    char* buf = 0;
    bool loop = true;
    while(loop) {
        delete [] buf;
        bufSize <<= 1;
        buf = new char[bufSize];

        errno = 0;
        if(strerror_r(errnum, buf, bufSize)) {
            switch(errno) {
                case ERANGE:
                    break; // go through again
                case EINVAL:
                    o << L"(unknown -- invalid error code)";
                    loop = false;
                    break;
                default:
                    o << L"(unknown -- error " << errno
                      << L" during strerror_r conversion)";
                    loop = false;
                    break;
            }
        } else {
            // conversion was OK
            try {
                o << utf8ToUcs4(buf);
            }
            catch(...) {
                delete [] buf;
                o << "(invalid UTF-8 std::string)";
            }
            loop = false;
        }
    }

    delete [] buf;
#else
    // this is needed for the GNU version of strerror_r(), with
    // signature:
    //     char* strerror_r(int, char*, int);
    //
    // see implementation at glibc/sysdeps/generic/_strerror.c
    char buf[30];
    o << utf8ToUcs4(strerror_r(errnum, buf, sizeof(buf)), true);
#endif

    return o.str();
}



SystemError::SystemError()
    : Exception(errnoToString(errno))
{
}



SystemError::SystemError(int errnum)
    : Exception(errnoToString(errnum))
{
}



}
