/* lw-support/src/lib/Exceptions/SystemError.h
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



/*! \brief System call error.

This exception is used whenever a system call results in an error that
cannot be handled by LW Support (for instance, if an I/O device fails to
open). Many of the important cases will be handled with their own,
specific exception classes; those that aren't will be handled with this
class.

This class is intended as a direct replacement for the global variable
\a errno.

*/
class SystemError : public Exception {
private:
    static std::wstring errnoToString(int errnum);

public:
    /*! \brief Construct from errno.

    This constructor will examine the current value of \a errno and
    build the exception from there. The exception is always built, even
    if \a errno is 0.

    */
    SystemError();

    /*! \brief Construct from error number.

    \param errnum The error number.

    This constructor doesn't use the global \a errno variable, but
    instead uses the user-supplied value. The exception is always built,
    even if \a errnum is 0.

    */
    SystemError(int errnum);
};



}
