/* lw-support/src/lib/IO/Device.h
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



/*! \brief Base I/O device class.

This class is the base of the I/O device hierarchy. It specifies the
common functionality, such as closing, reading and writing. It provides
for both blocking and non-blocking I/O (which means that an 'open'
function would have to open its file descriptor in non-blocking mode).

*/
class IODevice : public IOInterface {
protected:
    friend class EventManager;

    /// Returns a file descriptor suitable for EventManager.
    virtual int getListenFd() const = 0;

public:
    /// Constructor. Does nothing.
    IODevice()
    { }

    /// Destructor. Calls close().
    virtual ~IODevice()
    { }



    /*! \brief Closes the device.

    \throws SystemError if a system error occurs.

    This call will close a device.

    */
    virtual void close() = 0;

    // implemented virtuals
    virtual IOMode getIOMode() const
    { return ioMode; }
};



}
