/* lw-support/src/lib/IO/Pipe.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



Pipe::Pipe()
    : read(0), write(0)
{
    int fds[2];
    if(TEMP_FAILURE_RETRY(::pipe(fds)))
        throw SystemError()
            .chain(L"Pipe::Pipe()").chain(L"pipe()");
    try {
        read = new EndPipe(this, fds[0], true);
        read->setNonBlocking();
        write = new EndPipe(this, fds[1], false);
        write->setNonBlocking();
    }
    catch(...) {
        delete read;
        delete write;
        throw;
    }
}



Pipe::~Pipe()
{
    delete read;
    delete write;
}



EndPipe::EndPipe(Pipe* myPipe, int fd, bool read)
    : myPipe(myPipe)
{
    this->fd = fd;
    this->ioMode = read ? IORead : IOWrite;
}



EndPipe::~EndPipe()
{
    // unregister with the containing Pipe object
    if(ioMode == IORead) myPipe->read = 0;
    else myPipe->write = 0;

    // closing happens automatically in IOPosixDevice's destructor.
}



}
