/* lw-support/src/lib/IO/Pipe.h
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



/*! \brief Pipe class container.

This class, when instantiated, creates a POSIX pipe device. This device
has two ends (of type EndPipe); you can write to one of these ends, and
read from the other.

This class just acts as a container for the two ends of the pipe.
Deleting it will delete the two end pipes.

*/
class Pipe : public Uncopyable {
private:
    friend class EndPipe;

public:
    /*! \brief Constructor.

    \throws lw::SystemError if an error occurs while creating the pipes.

    The constructor sets up both ends of the pipe.

    */
    Pipe();

    /// Destructor. Closes both ends of the pipe.
    ~Pipe();

    /// Reading end of the pipe.
    EndPipe* read;

    /// Writing end of the pipe.
    EndPipe* write;
};



/*! \brief One end of a pipe connection.

See the lw::Pipe class for a better description of POSIX pipes. This
object has a private constructor, since it can only be created as part
of an lw::Pipe container object.

The destructor will automatically unregister the object with its
containing lw::Pipe, in order to avoid double deletion when the Pipe
instance is deleted.

*/
class EndPipe : public IOPosixDevice {
private:
    friend class Pipe;
    Pipe* myPipe;
    EndPipe(Pipe* myPipe, int fd, bool read);

public:
    virtual ~EndPipe();
};



}
