/* lw-support/src/lib/IO/SerialPort.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



void IOSerialPort::open(IOMode mode, const char* path, int baud, bool twoStopBits)
{
    if(fd != -1) close();

    try {
        posixOpen(path, mode);
        setOptions(baud, twoStopBits);
    }
    catch(Exception& e) {
        e.chain(L"IOSerialPort::open()");
        close();
        throw;
    }
}



void IOSerialPort::setOptions(int baud, bool twoStopBits)
{
    // read current characteristics
    struct termios term;
    if(tcgetattr(fd, &term) < 0) {
        throw SystemError()
            .chain(L"tcgetattr()")
            .chain(L"IOSerialPort::setOptions()");
    }

    // set 8N1 mode
    term.c_iflag &= ~(IGNBRK | BRKINT | PARMRK | ISTRIP | INLCR | IGNCR
        | ICRNL | IXON | IXOFF);
    term.c_oflag &= ~OPOST;
    term.c_cflag &= ~(PARENB | CSIZE | CSTOPB);
    term.c_cflag |= CS8;
    term.c_lflag &= ~(ECHO | ECHONL | ICANON | ISIG | IEXTEN);

    // set baud rate and stop bits
    if(twoStopBits) term.c_cflag |= CSTOPB;
    if(cfsetspeed(&term, baud)) {
        throw SystemError()
            .chain(L"cfsetspeed()")
            .chain(L"IOSerialPort::setOptions()");
    }

    // set attributes
    if(tcsetattr(fd, TCSAFLUSH, &term)) {
        throw SystemError()
            .chain(L"tcsetattr()")
            .chain(L"IOSerialPort::setOptions()");
    }

}



void IOSerialPort::discardInput()
{
    if(TEMP_FAILURE_RETRY(tcflush(fd, TCIFLUSH))) {
        throw SystemError()
            .chain(L"tcflush()")
            .chain(L"IOSerialPort::discardInput()");
    }
}



}
