/* lw-support/src/lib/IO/SerialPort.h
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



/*! \brief Serial port I/O device.

This class allows you to control a serial (RS232) port or equivalent
device. It provides control over the baud rate and whether you want to
use one stop bit (the standard) or two (useful for communicating with
embedded devices when the baud rate generators don't exactly match). It
does not provide support for parity, flow control or other than 8 data
bits.

*/
class IOSerialPort : public IOPosixDevice {
public:
    /// Constructor (doesn't open any device).
    IOSerialPort()
    { }



    /*! \brief Open or reopen device.

    \param mode The mode in which the device should be opened.
    \param path Path to the device node to open.
    \param baud The baud rate you want.
    \param twoStopBits \a true if you want two stop bits; \a false if
        you don't.
    \throws IOModeError if you pass lw::IOClosed as the mode.
    \throws SystemError if the device couldn't be opened or if the
        options cannot be set.

    Opens the device, setting the desired baud rate and number of stop
    bits. If a device is already open, it is closed first.

    */
    void open(IOMode mode, const char* path, int baud,
        bool twoStopBits = false);



    /*! \brief Change device options (baud rate etc.).

    \param baud The new baud rate.
    \param twoStopBits \a true if you want two stop bits; \a false if
        you don't.
    \throws IOModeError if the device is closed.
    \throws SystemError if the options cannot be set.

    Changes the baud rate and number of stop bits.

    */
    void setOptions(int baud, bool twoStopBits = false);



    /// Discard any input waiting to be read.
    void discardInput();
};



}
