/* lw-support/src/lib/Net/Address/IPv6.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



namespace {

uint8_t ipv6_any_initialiser[16] = {
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};
uint8_t ipv6_loopback_initialiser[16] = {
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1
};

}



const NetAddressIPv6 NetAddressIPv6::any(ipv6_any_initialiser);
const NetAddressIPv6 NetAddressIPv6::loopback(ipv6_loopback_initialiser);



std::wstring NetAddressIPv6::toString() const
{
    char buffer[50]; // should be enough for a full IPv6 address
    if(!inet_ntop(AF_INET6, addr, buffer, sizeof(buffer))) {
        throw lw::ProgramException().chain(L"NetAddressIPv6::toString()");
    }

    return lw::strToUcs4(buffer);
}



NetAddressIPv6 NetAddressIPv6::fromString(const std::wstring& str)
{
    std::string ip;
    try {
        ip = ucs4ToAscii(stripWhitespace(str));
    }
    catch(lw::Exception& e) {
        e.chain(L"NetAddressIPv6::fromString()");
        throw;
    }

    struct in6_addr addr;

    if(!inet_pton(AF_INET6, ip.c_str(), &addr))
        throw ParseError(L"Not an IPv6 address.", str, 0);

    return NetAddressIPv6(addr.s6_addr);
}



NetAddressIPv6 NetAddressIPv6::fromDNS(const std::string& host)
{
    // attempt to convert from numeric form first
    try {
        return fromString(lw::asciiToUcs4(host));
    }
    catch(...) { }

    // now perform blocking DNS lookup
    struct hostent he, * hep = 0;
    int err = 0, result = 0;
    char buffer[4096];

    result = gethostbyname2_r(host.c_str(), AF_INET6,
        &he, buffer, sizeof(buffer), &hep, &err);
    switch(result) {
        case 0:
            // success!
            break;

        case HOST_NOT_FOUND:
            throw DnsError(L"Host not found.", host);

        case TRY_AGAIN:
            throw DnsError(L"Temporary failure in name resolution.", host);

        case NO_RECOVERY:
            throw DnsError(L"Unrecoverable failure in name resolution.", host);

        case NO_ADDRESS:
            throw DnsError(L"Host does not have an associated IPv6 address.", host);

        default:
            throw DnsError(L"Unknown DNS error.", host);
    }


    return NetAddressIPv6(he.h_addr);
}



}
