/* lw-support/src/lib/Net/Client.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



void NetClient::checkErrors()
{
    int error;
    socklen_t socklen = sizeof(error);

    if(getsockopt(fd, SOL_SOCKET, SO_ERROR, &error, &socklen) == -1)
        throw SystemError().chain(L"NetClient::checkSocketErrors()")
                           .chain(L"getsockopt()");

    if(error)
        throw SystemError(error).chain(L"NetClient::checkSocketErrors()")
                                .chain(L"(pending error)");
}



void NetClient::shutdown(bool read, bool write)
{
    if(ioMode == IOClosed)
        throw IOModeError(this, L"shutdown", IONone);

    IOMode newIoMode;
    int shut;

    if(read && write) {
        newIoMode = IONone;
        shut = SHUT_RDWR;
    } else if (read) {
        if(ioMode == IOWrite || ioMode == IOReadWrite) newIoMode = IOWrite;
        else newIoMode = IONone;
        shut = SHUT_RD;
    } else if (write) {
        if(ioMode == IORead || ioMode == IOReadWrite) newIoMode = IORead;
        else newIoMode = IONone;
        shut = SHUT_WR;
    } else {
        return;
    }

    if(TEMP_FAILURE_RETRY(::shutdown(fd, shut)))
        throw SystemError().chain(L"NetClient::shutdown()");
    ioMode = newIoMode;
}



}
