/* lw-support/src/lib/Thread/CompletionNotifier.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



CompletionNotifier::~CompletionNotifier()
{
    pthread_cond_broadcast(&cond);
    while(true) {
        pthread_mutex_lock(&mut);
        if(!pthread_cond_destroy(&cond)) break;
        pthread_mutex_unlock(&mut);
    }
    pthread_mutex_unlock(&mut);
    pthread_mutex_destroy(&mut);
}



void CompletionNotifier::wait(int maxTime)
{
    if(maxTime < 0) {
        wait();
        return;
    }

    struct timespec ts;
    ts.tv_sec = maxTime / 1000;
    ts.tv_nsec = (maxTime % 1000) * 1000000;
    pthread_mutex_lock(&mut);

    while(!done) {
        switch(pthread_cond_timedwait(&cond, &mut, &ts)) {
            case ETIMEDOUT: {
                pthread_mutex_unlock(&mut);
                throw Timeout();
            }

            case 0: break;
        }
    }
    pthread_mutex_unlock(&mut);
}



}
