/* lw-support/src/lib/Thread/Mutex.h
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



/*! \brief Mutex.

This object may be one of two types: fast or recursive. A fast mutex
cannot be locked twice in the same thread. Attempting to do so will
result in undefined behaviour (most likely deadlock).

However, a recursive mutex can be locked twice in the same thread. It
must be unlocked an equal number of times. It will only become available
to other threads when each lock() call has been matched by an unlock()
call.

\warning Behaviour is undefined if a locked mutex is destroyed.
         Behaviour is undefined if a thread which does not own the lock
         tries to unlock the mutex.

*/
class Mutex {
private:
    pthread_mutex_t mut;

public:
    /*! \brief Constructor.

    \param recursive Indicates whether the mutex should be fast or
                     recursive.

    Creates the mutex object. Note that, once constructed, the type of
    a mutex cannot be changed. Mutexes are always constructed unlocked.

    */
    Mutex(bool recursive = false);



    /// Destructor.
    ~Mutex();



    /// Lock the mutex.
    inline void lock()
    {
        pthread_mutex_lock(&mut);
    }



    /// Unlock the mutex.
    inline void unlock()
    {
        pthread_mutex_unlock(&mut);
    }
};



}
