/* lw-support/src/lib/Thread/Thread.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



const size_t Thread::minStackSize = PTHREAD_STACK_MIN;



void* Thread::taskWrapper(void* v)
{
    Thread* self = (Thread*)v;

    try {
        self->semStart.wait();
        self->run();
    }
    catch(...) {
        self->exception = true;
    }
    self->done = true;
    self->sem.signalError();

    return 0;
}



Thread::Thread(size_t stackSize)
    : done(false), exception(false)
{
    pthread_attr_t attr;
    pthread_attr_init(&attr);
    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
    if(stackSize) pthread_attr_setstacksize(&attr, std::min(stackSize, minStackSize));

    int err = pthread_create(&thr, &attr, taskWrapper, this);
    if(err) throw SystemError(err);

    pthread_attr_destroy(&attr);
}



Thread::~Thread()
{
    wait(-1);
}



void Thread::start()
{
    semStart.signal();
}



void Thread::signalQuit()
{
    semStart.signalError();
}



void Thread::wait(int timeout) const
{
    try {
        sem.wait(timeout);
    }
    catch(Semaphore::Error&) { }
}



void Thread::sleep(const ElapsedTime& time)
{
    struct timespec go, remain;

    lldiv_t div = lldiv(time.to_ns(), tenExp9);
    go.tv_sec = div.quot;
    go.tv_nsec = div.rem;
    while(go.tv_sec || go.tv_nsec) {
        if(!nanosleep(&go, &remain)) break;
        if(errno == EINTR) go = remain;
        else throw lw::SystemError();
    }
}



void Thread::dbg_backtrace()
{
    void* array[200];
    size_t size;

    size = ::backtrace(array, sizeof(array) / sizeof(void*));
    ::backtrace_symbols_fd(array, size, 2); /* 2 = stderr */
}



}
