/* lw-support/src/lib/Time/DateTime.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



std::wstring DateTime::toString(bool extended, bool decimalTime,
    ISO8601Format dateFormat, ISO8601Precision precision) const
{
    std::wostringstream o;
    switch(precision) {
        case ISO8601PrecisionYear:
        case ISO8601PrecisionWeek:
        case ISO8601PrecisionMonth:
        case ISO8601PrecisionDay:
            o << date.toString(extended, dateFormat, precision);
            break;

        case ISO8601PrecisionHour:
        case ISO8601PrecisionMinute:
        case ISO8601PrecisionSecond:
        case ISO8601PrecisionFull:
            o << date.toString(extended, dateFormat, ISO8601PrecisionFull);
            o << L'T';
            o << time.toString(extended, precision, decimalTime);
            break;
    }
    return o.str();
}



DateTime DateTime::fromString(const std::wstring& str,
    ISO8601Format dateFormat)
{
    std::wstring::size_type timeStart = str.find('T');
    if(timeStart == std::string::npos) {
        return DateTime(Date::fromString(str, dateFormat));
    }
    return DateTime(
        Date::fromString(str.substr(0, timeStart), dateFormat),
        DayTime::fromString(str.substr(timeStart + 1, std::wstring::npos))
    );
}



DateTime& DateTime::addNanoSeconds(int64_t ns)
{
    int dayDelta;
    time.addNanoSeconds(ns, dayDelta);
    date += dayDelta;
    return *this;
}



DateTime& DateTime::addSeconds(int s)
{
    int dayDelta;
    time.addSeconds(s, dayDelta);
    date += dayDelta;
    return *this;
}



DateTime& DateTime::addMinutes(int m)
{
    int dayDelta;
    time.addMinutes(m, dayDelta);
    date += dayDelta;
    return *this;
}



DateTime& DateTime::addHours(int h)
{
    int dayDelta;
    time.addHours(h, dayDelta);
    date += dayDelta;
    return *this;
}



DateTime& DateTime::addDays(int d)
{
    date += d;
    return *this;
}



std::wostream& operator<<(std::wostream& ostr, const DateTime& d)
{
    ostr << d.toString();
    return ostr;
}



}
