/* lw-support/src/lib/Time/ElapsedTime.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



std::wstring ElapsedTime::toString(int decPlaces, bool exp) const
{
    std::wostringstream o;

    o << std::setprecision(decPlaces);

    if(exp) {
        o << std::scientific;
        o << (time / 1e9) << 's';

    } else {
        o << std::fixed;
        if(time < 100) {
            o << time << "ns";

        } else if(time < 1e5) {
            o << (time / 1e3) << "us";

        } else if(time < 1e8) {
            o << (time / 1e6) << "ns";

        } else {
            o << (time / 1e9) << 's';

        }
    }

    return o.str();
}



ElapsedTime ElapsedTime::fromString(const std::wstring& str)
{
    std::wstring::size_type strSize = str.length();
    std::wistringstream istr(str);
    double d;

    istr >> d >> std::ws;
    int afterNumPos = (strSize - istr.rdbuf()->in_avail());
    if(istr.bad() || istr.fail()) {
        throw ParseError(L"Not a number.", str, afterNumPos);
    }

    if(d < 0 || !isfinite(d)) {
        throw ParseError(L"Not a valid time period.", str, afterNumPos);
    }

    if(!istr.eof()) {
        std::wstring unit;
        istr >> unit >> std::ws;

        if(unit == L"s") {
            d *= 1e9;

        } else if(unit == L"ms") {
            d *= 1e6;

        } else if(unit == L"us") {
            d *= 1e3;

        } else if(unit == L"ns") {
            // nothing to do

        } else {
            throw ParseError(L"Unrecognised unit.", str, afterNumPos);
        }

        if(!istr.eof()) {
            throw ParseError(L"Data after unit.", str,
                (strSize - istr.rdbuf()->in_avail()));
        }
    }

    return ElapsedTime((uint64_t)(d), true);
}



ElapsedTime::ElapsedTime(struct timeval* tv)
    : time(tv->tv_sec * tenExp9 + tv->tv_usec * tenExp3)
{
    if(tv->tv_sec < 0 || tv->tv_usec < 0 || tv->tv_usec >= tenExp6) {
        throw BadTimeValue();
    }
}



ElapsedTime::ElapsedTime(struct timespec* ts)
    : time(ts->tv_sec * tenExp9 + ts->tv_nsec)
{
    if(ts->tv_sec < 0 || ts->tv_nsec < 0 || ts->tv_nsec >= tenExp9) {
        throw BadTimeValue();
    }
}



std::wostream& operator<<(std::wostream& ostr, const ElapsedTime& t)
{
    ostr << t.toString();
    return ostr;
}



}
