/* lw-support/src/lib/Time/StopWatch.h
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



/*! \brief Stop watch timer class.

This is a simple stopwatch class, which uses the ElapsedTime class to
record elapsed time to the nearest nanosecond (ostensibly; this depends
on operating system support).

\sa ElapsedTime

*/
class StopWatch {
private:
    struct timespec started;
    mutable struct timespec stopped;
    bool hasStarted;

public:
    /// Constructor. Starts timer.
    StopWatch();

    /// Starts (or restarts) the timer.
    void start();

    /// Stops the timer.
    void stop();

    /*! \brief Retrieves elapsed time.

    \returns The elapsed time.

    If the timer is currently running, it returns the time elapsed since
    it was started. If the timer is stopped, it returns the time elapsed
    between starting (or restarting) and stopping the timer.

    */
    ElapsedTime elapsed() const;
};



}
