/* lw-support/src/lib/Util/Completion.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



CompletionTask::CompletionTask(CompletionCallback* callback)
    : callback(callback)
{
}



CompletionTask::~CompletionTask()
{
    if(callback) {
        try {
            callback->aborted(this, lw::Exception(L"Task still active when destructor called."));
        }
        catch(...) { }
    }
}



void CompletionTask::taskCompleted()
{
    CompletionCallback* cb = callback;
    if(!callback) throw TaskAlreadyCompleted();
    callback = 0;
    cb->completed(this);
}



void CompletionTask::taskAborted(const Exception& e)
{
    CompletionCallback* cb = callback;
    if(!callback) throw TaskAlreadyCompleted();
    callback = 0;
    cb->aborted(this, e);
}



CompletionList::CompletionList()
    : abortingTask(0)
{
}



CompletionList::~CompletionList()
{
    abortAll();
}



void CompletionList::abortAll()
{
    while(!tasks.empty()) {
        abortingTask = tasks.front();
        tasks.pop_front();
        try {
            abortingTask->taskAbort(lw::Exception(L"Task list shutdown."));
        }
        catch(...) { }
        if(abortingTask) dbg_taskNotAborted(abortingTask);
    }
}



void CompletionList::registerTask(CompletionTask* task)
{
    NullPointer::check(task, L"task", L"CompletionList::registerTask()");
    tasks.push_back(task);
}



bool CompletionList::empty()
{
    return tasks.empty();
}



void CompletionList::completed(CompletionData* data)
{
    CompletionTask* task = dynamic_cast<CompletionTask*>(data);
    if(!task) {
        dbg_dataNotTask(data);
        return;
    }
    taskIter iter = find(tasks.begin(), tasks.end(), data);
    if(iter == tasks.end()) {
        dbg_taskNotActive(task);
        return;
    }
    tasks.erase(iter);
}



void CompletionList::aborted(CompletionData* data, const lw::Exception&)
{
    if(abortingTask == data) abortingTask = 0;
    completed(data);
}



void CompletionList::dbg_dataNotTask(CompletionData* data)
{
    stderr << lw::Log::start(L"[DEBUG] CompletionList")
        << L"Data passed to CompletionList::completed() or aborted() was not derived from\n"
        << L"CompletionTask, as required."
        << L"\n  this = "
        << this
        << L"\n  data = "
        << data
        << lw::Log::end;
}



void CompletionList::dbg_taskNotAborted(CompletionTask* task)
{
    stderr << lw::Log::start(L"[DEBUG] CompletionList")
        << L"Task not aborted."
        << L"\n  this = "
        << this
        << L"\n  task = "
        << task
        << lw::Log::end;
}



void CompletionList::dbg_taskNotActive(CompletionTask* task)
{
    stderr << lw::Log::start(L"[DEBUG] CompletionList")
        << L"Task passed to CompletionList::completed() or aborted() not active."
        << L"\n  this = "
        << this
        << L"\n  task = "
        << task
        << lw::Log::end;
}



}
