/* lw-support/src/lib/Util/FIFO.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lw {



void FIFO::reallocBuffer(size_t required)
{
    // sanity checks
    if(required < bufferSize) return;
    if(required > maxSize) throw FIFOOverflow();

    // try to reduce frequency of small allocations
    required = std::min(required + 128, maxSize);

    // try to allocate memory
    char* newBuffer = 0;
    try {
        newBuffer = new char[required];
    }
    catch(std::bad_alloc&) {
        throw FIFOOutOfMemory(required);
    }

    // copy across old data
    if(data) {
        memcpy(newBuffer, data, dataSize);
        data = newBuffer;
    }

    // free old memory
    delete [] buffer;
    buffer = newBuffer;
    bufferSize = required;
}



}
