/* lw-support/src/tests/Log.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

#include "lw-support"
#include <iostream>
#include <iomanip>
#include <stdexcept>



const char hexData[] = {
    0, 1, 2, 3, 4, 5, 6, 7,
    100, 101, 102, 103, 104, 105, 106, 107,
    200, 201, 202, 203, 204, 205, 206, 207,
    0
};

const char asciiData[] = "Hello,\tworld!\r\nMore data on a new line\n";



int main(int argc, char* argv[])
{
    if(argc == 2 && !strcmp(argv[1], "--print-summary")) {
        std::wcout << L"Tests the 'Log' object.\n";
        return 0;
    }

    if(argc > 2) {
        std::wcerr << L"Expecting no arguments (logs to screen) or one (logs to filename).\n";
        return 1;
    }

    int ret = 0;
    lw::Log* log = 0;
    try {
        if(argc == 1) log = new lw::Log(std::wcout, true);
        else log = new lw::Log(argv[1], true);

        *log << lw::Log::start(L"Logging test")
            << L"Logging object instantiated.\n"
            << "This is some ASCII data.\n"
            << L"And a number: "
            << std::setfill(L'0')
            << std::setw(4)
            << std::hex
            << 255
            << lw::Log::end;

        *log << lw::Log::start(L"Hex Dump")
            << lw::Log::hexDump(hexData, sizeof(hexData))
            << lw::Log::end;

        *log << lw::Log::start(L"ASCII Dump")
            << lw::Log::asciiDump(asciiData, sizeof(asciiData))
            << lw::Log::end;

        *log << lw::Log::start(L"Exceptions")
            << L"std::out_of_bounds\n"
            << std::out_of_range("exception test")
            << L"\n\nlw::Exception\n"
            << lw::Exception(L"A test of the log system")
            << L"\n\nlw::ProgramException (derived from lw::Exception)\n"
            << lw::ProgramException()
            << lw::Log::end;

        *log << lw::Log::start(L"Network addresses")
            << L"lw::NetAddressIPv4\n"
            << lw::NetAddressIPv4::any
            << L"\n\nlw::NetAddressIPv6\n"
            << lw::NetAddressIPv6::any
            << lw::Log::end;
    }
    catch(lw::Log::BoxException& e) {
        std::wcerr << L"Boxing exception.\n";
        ret = 1;
    }
    catch(lw::Exception& e) {
        std::wcerr << e.toString() << std::endl;
        ret = 1;
    }
    catch(std::exception& e) {
        std::wcerr << lw::utf8ToUcs4(e.what(), true) << std::endl;
        ret = 1;
    }
    catch(...) {
        std::wcerr << L"Unknown exception caught.\n";
        ret = 1;
    }

    delete log;
    return ret;
}
