/* lw-support/src/tests/ThreadedPipe.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

#include "lw-support"
using namespace lw;

#include <iostream>



const char* message = "Hello, world!\n";
size_t messageLen = strlen(message) + 1;



/// Convenience function: print the current time followed by a colon.
inline std::wostream& printTime()
{
    return std::wcout << DayTime::now()
        .toString(true, ISO8601PrecisionFull, true)
        << L": ";
}



class ReaderThread : public Thread {
private:
    EndPipe* read;

protected:
    virtual void run()
    {
        printTime() << L"Reading message\n";

        char buffer[messageLen];
        read->readBlock(buffer, messageLen);

        if(memcmp(buffer, message, messageLen)) printTime() << L"Message mismatch!\n";
        else printTime() << L"Message received\n";
    }

public:
    ReaderThread(EndPipe* read)
        : read(read)
    {
        start();
    }

    virtual ~ReaderThread()
    {
        printTime() << L"Deleting ReaderThread object [waiting]\n";
        wait(-1);
        printTime() << L"Deleting ReaderThread object [now]\n";
    }
};



class WriterThread : public Thread {
private:
    EndPipe* write;

protected:
    virtual void run()
    {
        sleep(ElapsedTime::from_ms(1000));
        printTime() << L"Writing message\n";

        write->writeBlock(message, messageLen);
    }

public:
    WriterThread(EndPipe* write)
        : write(write)
    {
        start();
    }

    virtual ~WriterThread()
    {
        printTime() << L"Deleting WriterThread object [waiting]\n";
        wait(-1);
        printTime() << L"Deleting WriterThread object [now]\n";
    }
};



int main(int argc, char* argv[])
{
    if(argc == 2 && !strcmp(argv[1], "--print-summary")) {
        std::wcout << L"POSIX pipe and non-blocking threaded I/O regression test.\n";
        return 0;
    }

    if(argc != 1) {
        std::wcerr << L"Not expecting any arguments.\n";
        return 1;
    }

    int ret = 0;
    Pipe* p = 0;
    Thread* thr1 = 0, * thr2 = 0;
    try {
        p = new Pipe;
        thr1 = new ReaderThread(p->read);
        thr2 = new WriterThread(p->write);
    }
    catch(Exception& e) {
        std::wcerr << e.toString() << std::endl;
        ret = 1;
    }
    catch(std::exception& e) {
        try {
            std::wcerr << utf8ToUcs4(e.what()) << std::endl;
        }
        catch(...) {
            std::wcerr << L"std::exception caught but couldn't display.\n";
        }
        ret = 1;
    }
    catch(...) {
        std::wcerr << L"Unknown exception caught.\n";
        ret = 1;
    }

    delete thr1;
    delete thr2;
    delete p;

    return ret;
}
