/* lw-support/src/tests/Time_Date.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

#include "lw-support"
using namespace lw;

#include <iostream>
#include <string>
using namespace std;



const Date Epoch(1970, 1 ,1);



void toString(const Date& d)
{
    wchar_t opt;
    wcout << L"Extended (y/n)? ";
    wcin >> opt;
    bool extended = (opt == 'y');

    wcout << L"\nFormat: [c]alendar, [o]rdinal or [w]eek date? ";
    wcin >> opt;
    ISO8601Format format = ISO8601FormatDetect;
    switch(opt) {
        case 'c':
            format = ISO8601FormatCalendar;
            break;

        case 'o':
            format = ISO8601FormatOrdinal;
            break;

        case 'w':
            format = ISO8601FormatWeekDate;
            break;
    }

    wcout << L"Precision ([f]ull, [y]ear, [w]eek, m[o]nth, [d]ay, [h]our, [m]inute, [s]econd)? ";
    wcin >> opt;
    ISO8601Precision precision = ISO8601PrecisionFull;
    switch(opt) {
        case 'y':
            precision = ISO8601PrecisionYear;
            break;

        case 'w':
            precision = ISO8601PrecisionWeek;
            break;

        case 'o':
            precision = ISO8601PrecisionMonth;
            break;

        case 'd':
            precision = ISO8601PrecisionDay;
            break;

        case 'h':
            precision = ISO8601PrecisionHour;
            break;

        case 'm':
            precision = ISO8601PrecisionMinute;
            break;

        case 's':
            precision = ISO8601PrecisionSecond;
            break;
    }

    wcout << d.toString(extended, format, precision) << endl;
}



void add(Date& d)
{
    int amt;
    wcout << L"How many? ";
    wcin >> amt;

    d += amt;
}



bool innerTest(Date& d)
{
    wcout << endl;
    wcout << L"Year   : " << d.getYear() << endl;
    wcout << L"Month  : " << d.getMonth() << endl;
    wcout << L"Day    : " << d.getDay() << endl;
    wcout << L"Ordinal: " << d.getOrdinal() << endl;
    wcout << L"Weekday: " << d.getWeekDay() << endl;
    wcout << L"Week   : " << d.getWeek() << endl;
    wcout << L"dEpoch : " << (d - Epoch) << endl;

    wchar_t opt;
    wcout << L"To [s]tring (ISO8601), [a]dd days, [q]uit? ";
    wcin >> opt;

    switch(opt) {
        case 's':
            toString(d);
            return true;

        case 'a':
            add(d);
            return true;

        case 'q':
            return false;
    }

    return true;
}



Date fromString()
{
    wchar_t opt;
    wcout << L"\nFormat: [d]etect, [c]alendar, [o]rdinal or [w]eek date? ";
    wcin >> opt;
    ISO8601Format format = ISO8601FormatDetect;
    switch(opt) {
        case 'c':
            format = ISO8601FormatCalendar;
            break;

        case 'o':
            format = ISO8601FormatOrdinal;
            break;

        case 'w':
            format = ISO8601FormatWeekDate;
            break;
    }

    std::wstring str;
    wcout << L"Enter ISO8601 std::string: ";
    wcin >> str;

    return Date::fromString(str, format);
}



Date specify()
{
    wchar_t opt;
    wcout << L"\nFrom [o]rdinal, [w]eek or [m]onth+day? ";
    wcin >> opt;

    if(opt == 'o') {
        int year, ord;
        wcout << L"Enter year and ordinal: ";
        wcin >> year >> ord;
        return Date(year, ord);
    }

    if(opt == 'w') {
        int year, week, weekDay;
        wcout << L"Enter year, week and week day: ";
        wcin >> year >> week >> weekDay;
        return Date::fromWeek(year, week, weekDay);
    }

    int year, month, day;
    wcout << L"Enter year, month and day: ";
    wcin >> year >> month >> day;
    return Date(year, month, day);
}



bool runTest()
{
    wchar_t opt;
    wcout << L"\nFrom [s]tring (ISO8601), [c]urrent date, s[p]ecify or [q]uit? ";
    wcin >> opt;

    Date d(1970, 1, 1);

    switch(opt) {
        case 's':
            d = fromString();
            break;

        case 'c':
            d = Date::now();
            break;

        case 'p':
            d = specify();
            break;

        case 'q':
            return false;

        default:
            return true;
    }

    bool ok = true;
    do {
        try {
            ok = innerTest(d);
        }
        catch(Exception& e) {
            wcout << e.toString() << endl;
        }
    }while(ok);

    return true;
}



int main(int argc, char* argv[])
{
    if(argc == 2 && !strcmp(argv[1], "--print-summary")) {
        wcout << L"Test for the 'Date' object.\n";
        return 0;
    }

    bool ok = true;
    do {
        try {
            ok = runTest();
        }
        catch(Exception& e) {
            wcout << e.toString() << endl;
        }
    }while(ok);
}
