/* lw-support/src/tests/Time_DateTime.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

#include "lw-support"
using namespace lw;

#include <iostream>
#include <string>
using namespace std;



void toString(const DateTime& dt)
{
    wchar_t opt;
    wcout << L"Extended (y/n)? ";
    wcin >> opt;
    bool extended = (opt == 'y');

    wcout << L"Decimal time (y/n)? ";
    wcin >> opt;
    bool decimal = (opt == 'y');

    wcout << L"\nFormat: [c]alendar, [o]rdinal or [w]eek date? ";
    wcin >> opt;
    ISO8601Format format = ISO8601FormatDetect;
    switch(opt) {
        case 'c':
            format = ISO8601FormatCalendar;
            break;

        case 'o':
            format = ISO8601FormatOrdinal;
            break;

        case 'w':
            format = ISO8601FormatWeekDate;
            break;
    }

    wcout << L"Precision ([f]ull, [y]ear, [w]eek, m[o]nth, [d]ay, [h]our, [m]inute, [s]econd)? ";
    wcin >> opt;
    ISO8601Precision precision = ISO8601PrecisionFull;
    switch(opt) {
        case 'y':
            precision = ISO8601PrecisionYear;
            break;

        case 'w':
            precision = ISO8601PrecisionWeek;
            break;

        case 'o':
            precision = ISO8601PrecisionMonth;
            break;

        case 'd':
            precision = ISO8601PrecisionDay;
            break;

        case 'h':
            precision = ISO8601PrecisionHour;
            break;

        case 'm':
            precision = ISO8601PrecisionMinute;
            break;

        case 's':
            precision = ISO8601PrecisionSecond;
            break;
    }

    wcout << dt.toString(extended, decimal, format, precision) << endl;
}



void add(DateTime& dt)
{
    wchar_t which;
    wcout << L"Add [n]anoseconds, [s]econds, [m]inutes, [h]ours or [d]ays? ";
    wcin >> which;

    int amt;
    wcout << L"How many? ";
    wcin >> amt;

    switch(which) {
        case 'n':
            dt.addNanoSeconds(amt);
            break;

        case 's':
            dt.addSeconds(amt);
            break;

        case 'm':
            dt.addMinutes(amt);
            break;

        case 'h':
            dt.addHours(amt);
            break;

        case 'd':
            dt.addDays(amt);
            break;

        default:
            wcout << L"Not sure what to add.\n";
    }
}



bool innerTest(DateTime& dt)
{
    wcout << endl;
    wcout << L"Year   : " << dt.getDate().getYear() << endl;
    wcout << L"Month  : " << dt.getDate().getMonth() << endl;
    wcout << L"Day    : " << dt.getDate().getDay() << endl;
    wcout << L"Ordinal: " << dt.getDate().getOrdinal() << endl;
    wcout << L"Weekday: " << dt.getDate().getWeekDay() << endl;
    wcout << L"Week   : " << dt.getDate().getWeek() << endl;
    wcout << L"Hour   : " << dt.getTime().h() << endl;
    wcout << L"Minute : " << dt.getTime().m() << endl;
    wcout << L"Second : " << dt.getTime().s() << endl;
    wcout << L"Nano-s : " << dt.getTime().ns() << endl;

    wchar_t opt;
    wcout << L"To [s]tring (ISO8601), [a]dd, [q]uit? ";
    wcin >> opt;

    switch(opt) {
        case 's':
            toString(dt);
            return true;

        case 'a':
            add(dt);
            return true;

        case 'q':
            return false;
    }

    return true;
}



DateTime fromString()
{
    wchar_t opt;
    wcout << L"\nFormat: [d]etect, [c]alendar, [o]rdinal or [w]eek date? ";
    wcin >> opt;
    ISO8601Format format = ISO8601FormatDetect;
    switch(opt) {
        case 'c':
            format = ISO8601FormatCalendar;
            break;

        case 'o':
            format = ISO8601FormatOrdinal;
            break;

        case 'w':
            format = ISO8601FormatWeekDate;
            break;
    }

    std::wstring str;
    wcout << L"Enter ISO8601 std::string: ";
    wcin >> str;

    return DateTime::fromString(str, format);
}



DateTime specify()
{
    // get date
    Date d(1970, 1, 1);
    int year, week, weekDay, ord, month, day;

    wchar_t opt;
    wcout << L"\nFrom [o]rdinal, [w]eek or [m]onth+day? ";
    wcin >> opt;

    switch(opt) {
        case 'o':
            wcout << L"Enter year and ordinal: ";
            wcin >> year >> ord;
            d = Date(year, ord);
            break;

        case 'w':
            wcout << L"Enter year, week and week day: ";
            wcin >> year >> week >> weekDay;
            d = Date::fromWeek(year, week, weekDay);
            break;

        default:
            wcout << L"Enter year, month and day: ";
            wcin >> year >> month >> day;
            d = Date(year, month, day);
            break;
    }

    // get time
    int h, m, s, ns;
    wcout << L"\nEnter hour, minute, second, nanosecond: ";
    wcin >> h >> m >> s >> ns;

    return DateTime(d, DayTime(h, m, s, ns));
}



bool runTest()
{
    wchar_t opt;
    wcout << L"\nFrom [s]tring (ISO8601), [c]urrent date, s[p]ecify or [q]uit? ";
    wcin >> opt;

    DateTime dt(Date(1970, 1, 1));

    switch(opt) {
        case 's':
            dt = fromString();
            break;

        case 'c':
            dt = DateTime::now();
            break;

        case 'p':
            dt = specify();
            break;

        case 'q':
            return false;

        default:
            return true;
    }

    bool ok = true;
    do {
        try {
            ok = innerTest(dt);
        }
        catch(Exception& e) {
            wcout << e.toString() << endl;
        }
    }while(ok);

    return true;
}



int main(int argc, char* argv[])
{
    if(argc == 2 && !strcmp(argv[1], "--print-summary")) {
        wcout << L"Test for the 'DateTime' object.\n";
        return 0;
    }

    bool ok = true;
    do {
        try {
            ok = runTest();
        }
        catch(Exception& e) {
            wcout << e.toString() << endl;
        }
    }while(ok);
}
