/* lw-support/src/tests/Time_DayTime.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

#include "lw-support"
using namespace lw;

#include <iostream>
#include <string>
using namespace std;



void toString(const DayTime& t)
{
    wchar_t opt;
    wcout << L"Extended (y/n)? ";
    wcin >> opt;
    bool extended = (opt == 'y');

    wcout << L"Precision ([f]ull, [h]our, [m]inute, [s]econd)? ";
    wcin >> opt;
    ISO8601Precision precision = ISO8601PrecisionFull;
    switch(opt) {
        case 'h':
            precision = ISO8601PrecisionHour;
            break;

        case 'm':
            precision  = ISO8601PrecisionMinute;
            break;

        case 's':
            precision  = ISO8601PrecisionSecond;
            break;
    }

    wcout << L"Decimal (y/n)? ";
    wcin >> opt;
    bool decimal = (opt == 'y');

    wcout << t.toString(extended, precision, decimal) << endl;
}



void add(DayTime& t, wchar_t which)
{
    int dayDelta, amt;
    wcout << L"How many? ";
    wcin >> amt;

    switch(which) {
        case 'h':
            t.addHours(amt, dayDelta);
            break;

        case 'm':
            t.addMinutes(amt, dayDelta);
            break;

        case 'e':
            t.addSeconds(amt, dayDelta);
            break;

        case 'n':
            t.addNanoSeconds(amt, dayDelta);
            break;
    }

    wcout << L"Change in days: " << dayDelta << endl;
}



bool innerTest(DayTime& t)
{
    wcout << endl;
    wcout << L"Hours  : " << t.h() << endl;
    wcout << L"Minutes: " << t.m() << endl;
    wcout << L"Seconds: " << t.s() << endl;
    wcout << L"ns     : " << t.ns() << endl;

    wchar_t opt;
    wcout << L"To [s]tring (ISO8601), add [h]ours, add [m]inutes, add s[e]conds, add [n]s, [q]uit? ";
    wcin >> opt;

    switch(opt) {
        case 's':
            toString(t);
            return true;

        case 'h':
        case 'm':
        case 'e':
        case 'n':
            add(t, opt);
            return true;

        case 'q':
            return false;
    }

    return true;
}



DayTime fromString()
{
    std::wstring str;
    wcout << L"\nEnter ISO8601 std::string: ";
    wcin >> str;

    return DayTime::fromString(str);
}



DayTime specify()
{
    int h, m, s, ns;
    wcout << L"\nEnter hour, minute, second, nanosecond: ";
    wcin >> h >> m >> s >> ns;

    return DayTime(h, m, s, ns);
}



bool runTest()
{
    wchar_t opt;
    wcout << L"\nFrom [s]tring (ISO8601), [c]urrent time, s[p]ecify or [q]uit? ";
    wcin >> opt;

    DayTime t(0, 0, 0, 0);

    switch(opt) {
        case 's':
            t = fromString();
            break;

        case 'c':
            t = DayTime::now();
            break;

        case 'p':
            t = specify();
            break;

        case 'q':
            return false;

        default:
            return true;
    }

    bool ok = true;
    do {
        try {
            ok = innerTest(t);
        }
        catch(Exception& e) {
            wcout << e.toString() << endl;
        }
    }while(ok);

    return true;
}



int main(int argc, char* argv[])
{
    if(argc == 2 && !strcmp(argv[1], "--print-summary")) {
        wcout << L"Test for the 'DayTime' object.\n";
        return 0;
    }

    bool ok = true;
    do {
        try {
            ok = runTest();
        }
        catch(Exception& e) {
            wcout << e.toString() << endl;
        }
    }while(ok);
}
